/****************************************************************************
** Demonstration of drawing solid circles on the VGA screen                **
**  by Steven H Don                                                        **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************/
#include "dos.h"
#include "math.h"
#include "stdlib.h"

unsigned char *Screen = (unsigned char *)0xA0000000L;
unsigned char *Buffer;

//Switches to VGA 320x200 256 colour mode
void VGAScreen ()
{
  asm {
    mov ax, 0x13
    int 0x10
  }
}

//Switches back to text mode
void TextScreen ()
{
  asm {
    mov ax, 0x3
    int 0x10
  }
}

//Sets a pixel on the screen to a specific colour
void PutPixel (int x, int y, char c)
{
  Buffer [(y << 8) + (y << 6) + x] = c;
}

//Draws a horizontal line from (x1,y) to (x2,y)
void DrawHorLine (int x1, int y, int x2, char c)
{
  int offset, length;

  //Clip to vertical screen size
  if (y < 0 || y > 200) return;

  //Swap endpoints if necessary
  if (x1 > x2) { length = x2; x2 = x1; x1 = length; }

  //Clip to horizontal screen size
  if (x2 < 0 || x1 > 319) return;
  if (x1 < 0) x1 = 0;
  if (x2 > 319) x2 = 319;

  //Calculate address and length
  offset = (y << 8) + (y << 6) + x1;
  length = x2 - x1 + 1;

  //Draw the line
  memset (Buffer + offset, c, length);
}

//Draws a filled circle with the specified centerpoint and radius
void DrawSolidCircle (int centerx, int centery, int radius, char colour)
{
  int line, position;
  int x, y;
  long sqrad;

  //No radius is a single pixel
  if (radius < 1) {
    PutPixel (centerx, centery, colour);
    return;
  }

  //Square radius
  sqrad = radius * radius;

  //Center line
  DrawHorLine (centerx - radius, centery, centerx + radius, colour);

  //Rest of the circle
  for (line = 1; line < radius; line++) {
    //Calculate position
    position = sqrt (sqrad - (line * line));

    //Draw horizontal lines, taking advantage of circle's perfect symmetry
    DrawHorLine (centerx - position, centery - line, centerx + position, colour);
    DrawHorLine (centerx - position, centery + line, centerx + position, colour);
  }
}

void main()
{
  int CenterX, CenterY, Radius, Colour;

  //Initialize graphics
  VGAScreen ();

  //I'm drawing to the screen instead of a buffer
  Buffer = Screen;

  //Keep drawing circles until a keypress
  do {
    CenterX = random (320);
    CenterY = random (200);
    Radius = random (50);
    Colour = random (256);
    DrawSolidCircle (CenterX, CenterY, Radius, Colour);
  } while (!kbhit ());

  //Back to text
  TextScreen ();
}